unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, QDB, Name2Key;

type
  TMainForm = class(TForm)
    MainMenu        : TMainMenu;
    FileNewItem     : TMenuItem;
    FileOpenItem    : TMenuItem;
    FileSaveItem    : TMenuItem;
    FileCloseItem   : TMenuItem;
    FileExitItem    : TMenuItem;
    HelpMenu        : TMenuItem;
    HelpContentsItem: TMenuItem;
    StatusLine: TPanel;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Addresses: TQDB;
    SpeedBar: TPanel;
    FileNewSButton: TSpeedButton;
    FileOpenSButton: TSpeedButton;
    FileSaveSButton: TSpeedButton;
    FileCloseSButton: TSpeedButton;
    FileExitSButton: TSpeedButton;
    HelpContentsSButton: TSpeedButton;
    ListPanel: TPanel;
    KeyList: TListBox;
    FilterBox: TEdit;
    EntryPanel: TPanel;
    AddBox: TMemo;
    IsPerson: TCheckBox;
    HPhone: TEdit;
    WPhone: TEdit;
    FPhone: TEdit;
    E_Mail: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    ButtonPanel: TPanel;
    AddButton: TButton;
    ClearButton: TButton;
    DeleteButton: TButton;
    Image: TImage;
    Label7: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure FileNew(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FileSave(Sender: TObject);
    procedure FileClose(Sender: TObject);
    procedure FileExit(Sender: TObject);
    procedure HelpContents(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure KeyListClick(Sender: TObject);
    procedure FilterBoxExit(Sender: TObject);
  private
    procedure BlankEntries;
    procedure Activate;
    procedure DeActivate;
    procedure ScreenToItem(P: PChar; var PLen: word);
    procedure ItemToScreen(P: PChar);
    function  CurrentKey: TKey;
    procedure Update;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  MaxSize   = 400;   {largest item possible}
  BaseTitle = 'QDB Demo Address Book';

procedure TMainForm.FormCreate(Sender: TObject);
begin
Application.OnHint := ShowHint;
MainForm.Caption:=BaseTitle;
DeActivate;
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
StatusLine.Caption := Application.Hint;
end;

procedure TMainForm.FileNew(Sender: TObject);
begin
if SaveDialog.Execute then
  begin
  Addresses.FileName:=SaveDialog.FileName;
  MainForm.Caption:=BaseTitle + ' - ' + ExtractFileName(Addresses.FileName);
  Activate;
  end;
end;

procedure TMainForm.FileOpen(Sender: TObject);
begin
if OpenDialog.Execute then
  begin
  Addresses.FileName:=OpenDialog.FileName;
  BlankEntries;
  KeyList.Items:=Addresses.KeyList;
  if KeyList.Items.Count > 0 then
    begin
    KeyList.ItemIndex:=0;
    Update;
    end;
  MainForm.Caption:=BaseTitle + ' - ' + ExtractFileName(Addresses.FileName);
  Activate;
  end;
end;

procedure TMainForm.FileSave(Sender: TObject);
begin
Addresses.Save;
end;

procedure TMainForm.FileExit(Sender: TObject);
begin
if Addresses.FileName <> '' then Addresses.FileName:='';
Close;
end;

procedure TMainForm.HelpContents(Sender: TObject);
begin
Application.HelpCommand(HELP_CONTENTS, 0);
end;

procedure TMainForm.FileClose(Sender: TObject);
begin
Addresses.FileName:='';
BlankEntries;
KeyList.Items.Clear;
MainForm.Caption:=BaseTitle;
DeActivate;
end;

procedure TMainForm.ScreenToItem(P: PChar; var PLen: word);
var
  PP: PChar;
  PL: integer;
begin
PP:=P;
StrPCopy(PP,AddBox.Text);
inc(PP,StrLen(PP)+1);
if IsPerson.Checked then
  begin
  StrPCopy(PP,'Y');
  inc(PP,StrLen(PP)+1);
  end
else
  begin
  StrPCopy(PP,'N');
  inc(PP,StrLen(PP)+1);
  end;
StrPCopy(PP,HPhone.Text);
inc(PP,StrLen(PP)+1);
StrPCopy(PP,WPhone.Text);
inc(PP,StrLen(PP)+1);
StrPCopy(PP,FPhone.Text);
inc(PP,StrLen(PP)+1);
StrPCopy(PP,E_Mail.Text);
inc(PP,StrLen(PP)+1);
PLen:=PP - P + 1;
end;

procedure TMainForm.ItemToScreen(P: PChar);
var
  PP: PChar;
begin
Addresses.GetItem(P);
PP:=P;
AddBox.SetTextBuf(PP);
inc(PP,StrLen(PP)+1);
IsPerson.Checked:=(StrPas(PP)='Y');
inc(PP,StrLen(PP)+1);
HPhone.Text:=StrPas(PP);
inc(PP,StrLen(PP)+1);
WPhone.Text:=StrPas(PP);
inc(PP,StrLen(PP)+1);
FPhone.Text:=StrPas(PP);
inc(PP,StrLen(PP)+1);
E_Mail.Text:=StrPas(PP);
inc(PP,StrLen(PP)+1);
end;

procedure TMainForm.BlankEntries;
begin
AddBox.Text:='';
IsPerson.Checked:=true;
HPhone.Text:='';
WPhone.Text:='';
FPhone.Text:='';
E_Mail.Text:='';
AddBox.SetFocus;
end;

procedure TMainForm.Activate;
begin
AddButton.Enabled:=true;
ClearButton.Enabled:=true;
DeleteButton.Enabled:=true;
end;

procedure TMainForm.DeActivate;
begin
AddButton.Enabled:=false;
ClearButton.Enabled:=false;
DeleteButton.Enabled:=false;
end;

function TMainForm.CurrentKey: TKey;
var
  ThisKey : string;
  ThisItem: integer;
begin
if KeyList.Items.Count > 0 then
  begin
  ThisItem:=KeyList.ItemIndex;
  if ThisItem < 0 then
    begin
    Result:='';
    end
  else
    begin
    Result:=KeyList.Items[ThisItem];
    end;
  end
else
  begin
  Result:='';
  end;
end;

procedure TMainForm.AddButtonClick(Sender: TObject);
var
  NewKey : TKey;
  Item   : PChar;
  ItemLen: word;
begin
NewKey:=NameToKey(AddBox.Lines[0]);
if NewKey <> '' then
  begin
  if NewKey <> CurrentKey then {Add}
    begin
    Item:=StrAlloc(MaxSize);
    ScreenToItem(Item,ItemLen);
    Addresses.AddItem(Item,ItemLen,NewKey);
    StrDispose(Item);
    KeyList.Items:=Addresses.KeyList;
    KeyList.ItemIndex:=KeyList.Items.IndexOf(NewKey);
    end
  else                         {Change}
    begin
    Item:=StrAlloc(MaxSize);
    ScreenToItem(Item,ItemLen);
    if Addresses.ExactMatch(NewKey) then
      begin
      Addresses.ChangeItem(Item,ItemLen);
      end;
    StrDispose(Item);
    end;
  end
else
  begin
  MessageDlg('No address to add',mtInformation,[mbOK],0);
  end;
end;

procedure TMainForm.ClearButtonClick(Sender: TObject);
begin
BlankEntries;
end;

procedure TMainForm.DeleteButtonClick(Sender: TObject);
var
  ThisItem: integer;
  KeyToGo: string;
begin
ThisItem:=KeyList.ItemIndex;
if ThisItem <> -1 then
  begin
  KeyToGo:=KeyList.Items[ThisItem];
  if Addresses.ExactMatch(KeyToGo) then
    begin
    Addresses.DeleteItem;
    KeyList.Items:=Addresses.KeyList;
    end;
  end;
  if ThisItem >= KeyList.Items.Count then
    begin
    dec(ThisItem);
    end;
  KeyList.ItemIndex:=ThisItem;
  Update;
end;

procedure TMainForm.KeyListClick(Sender: TObject);
begin
Update;
end;

procedure TMainForm.Update;
var
  ThisKey : TKey;
  ThisItem: PChar;
begin
ThisKey:=CurrentKey;
if Addresses.ExactMatch(ThisKey) then
  begin
  ThisItem:=StrAlloc(Addresses.ItemSize);
  ItemToScreen(ThisItem);
  StrDispose(ThisItem);
  end
else
  begin
  BlankEntries;
  end;
end;



procedure TMainForm.FilterBoxExit(Sender: TObject);
begin
Addresses.Filter:=FilterBox.Text;
KeyList.Items:=Addresses.KeyList;
end;

end.
